@@ -15,6 +15,14 @@
 /*  `final' properties to an input string of character codes for the Arabic */
 /*  script.                                                                 */
 /*                                                                          */
+/*  This file is part of the FreeType project, and may only be used         */
+/*  modified and distributed under the terms of the FreeType project        */
+/*  license, LICENSE.TXT.  By continuing to use, modify, or distribute      */
+/*  this file you indicate that you have read the license and               */
+/*  understand and accept it fully.                                         */
+/*                                                                          */
+/* The file LICENSE.TXT can be found in the Pango distribution as           */
+/* pango/opentype/FT-license.txt                                            */
 /****************************************************************************/
 
 #include "arabic-ot.h"
@@ -33,7 +41,7 @@
  *
  */
 
-joining_class  arabic[] =
+static const joining_class  arabic[] =
 {
   /* U+0620 */
   none, none, right, right,
