@@ -435,32 +435,23 @@ make_langid (PangoLanguage *lang)
 #define CASEN(t,p) if (pango_language_matches (lang, t)) return MAKELANGID (LANG_##p, SUBLANG_NEUTRAL)
 
   /* Languages that most probably don't affect Uniscribe have been
-   * left out. Presumably still many of those mentioned below don't
-   * have any effect either.
+   * left out. Uniscribe is documented to use
+   * SCRIPT_CONTROL::uDefaultLanguage only to select digit shapes, so
+   * just leave languages with own digits.
    */
 
-  CASEN ("sq", ALBANIAN);
   CASEN ("ar", ARABIC);
-  CASEN ("az", AZERI);
-  CASEN ("hy", ARMENIAN);
   CASEN ("as", ASSAMESE);
   CASEN ("az", AZERI);
-  CASEN ("eu", BASQUE);
-  CASEN ("be", BELARUSIAN);
   CASEN ("bn", BENGALI);
-  CASEN ("bg", BULGARIAN);
-  CASEN ("ca", CATALAN);
   CASE ("zh-tw", CHINESE, TRADITIONAL);
   CASE ("zh-cn", CHINESE, SIMPLIFIED);
   CASE ("zh-hk", CHINESE, HONGKONG);
   CASE ("zh-sg", CHINESE, SINGAPORE);
   CASE ("zh-mo", CHINESE, MACAU);
-  CASEN ("hr", CROATIAN);
-  CASE ("sr", SERBIAN, CYRILLIC);
   CASEN ("dib", DIVEHI);
   CASEN ("fa", FARSI);
   CASEN ("ka", GEORGIAN);
-  CASEN ("el", GREEK);
   CASEN ("gu", GUJARATI);
   CASEN ("he", HEBREW);
   CASEN ("hi", HINDI);
@@ -487,13 +478,10 @@ make_langid (PangoLanguage *lang)
   CASEN ("tt", TATAR);
   CASEN ("te", TELUGU);
   CASEN ("th", THAI);
-  CASEN ("tr", TURKISH);
-  CASEN ("uk", UKRAINIAN);
   CASE ("ur-pk", URDU, PAKISTAN);
   CASE ("ur-in", URDU, INDIA);
   CASEN ("ur", URDU);
   CASEN ("uz", UZBEK);
-  CASEN ("vi", VIETNAMESE);
 
 #undef CASE
 #undef CASEN
@@ -524,7 +512,8 @@ dump_glyphs_and_log_clusters (gboolean rtl,
     printf ("  log_clusters: ");
     for (j = 0; j < itemlen; j++)
       printf ("%d ", log_clusters[j]);
-    printf ("\n");    nclusters = 0;
+    printf ("\n");
+    nclusters = 0;
     for (j = 0; j < itemlen; j++)
       {
 	if (j == 0 || log_clusters[j-1] != log_clusters[j])
@@ -566,7 +555,7 @@ dump_glyphs_and_log_clusters (gboolean rtl,
 	for (j = 0, charix = charix0; j < itemlen; j++, charix++)
 	  {
 	    if (j == 0 || log_clusters[j-1] != log_clusters[j])
-	      printf ("  Cluster %d: chars %d--",
+	      printf ("  Cluster %d: wchar_t %d--",
 		      clusterix, charix);
 	    if (j == itemlen - 1 || log_clusters[j] != log_clusters[j+1])
 	      {
@@ -590,8 +579,24 @@ dump_glyphs_and_log_clusters (gboolean rtl,
 
 #endif /* BASIC_WIN32_DEBUGGING */
 
+static int
+unichar_index (wchar_t *wtext,
+				int      ix)
+{
+  int i, index;
+
+  index = 0;
+  for (i = 0; i < ix; i++)
+    /* Ignore the low surrogate */
+    if (!(wtext[i] >= 0xDC00 && wtext[i] < 0xE000))
+      index++;
+
+  return index;
+}
+
 static void
-set_up_pango_log_clusters (gboolean rtl,
+set_up_pango_log_clusters (wchar_t *wtext,
+			   gboolean rtl,
 			   int      itemlen,
 			   WORD    *usp_log_clusters,
 			   int      nglyphs,
@@ -607,13 +612,13 @@ set_up_pango_log_clusters (gboolean rtl,
        * log_clusters array forwards.
        */
       int glyph0 = 0;
-      first_char_in_cluster = itemlen - 1;
+      first_char_in_cluster = unichar_index (wtext, itemlen - 1);
       for (j = itemlen - 1; j >= 0; j--)
 	{
 	  if (j < itemlen - 1 && usp_log_clusters[j+1] != usp_log_clusters[j])
 	    {
 	      /* Cluster starts */
-	      first_char_in_cluster = j;
+	      first_char_in_cluster = unichar_index (wtext, j);
 	    }
 	  if (j == 0)
 	    {
@@ -624,7 +629,7 @@ set_up_pango_log_clusters (gboolean rtl,
 	  else if (usp_log_clusters[j-1] == usp_log_clusters[j])
 	    {
 	      /* Cluster continues */
-	      first_char_in_cluster = j-1;
+	      first_char_in_cluster = unichar_index (wtext, j-1);
 	    }
 	  else
 	    {
@@ -646,7 +651,7 @@ set_up_pango_log_clusters (gboolean rtl,
 	  if (j > 0 && usp_log_clusters[j-1] != usp_log_clusters[j])
 	    {
 	      /* Cluster starts */
-	      first_char_in_cluster = j;
+	      first_char_in_cluster = unichar_index (wtext, j);
 	    }
 	  if (j == itemlen - 1)
 	    {
@@ -720,19 +725,24 @@ itemize_shape_and_place (PangoFont        *font,
   control.uDefaultLanguage = make_langid (analysis->language);
   state.uBidiLevel = analysis->level;
   
+#ifdef BASIC_WIN32_DEBUGGING
+  if (pango_win32_debug)
+    printf (G_STRLOC ": ScriptItemize: uDefaultLanguage:%04x uBidiLevel:%d\n",
+	    control.uDefaultLanguage, state.uBidiLevel);
+#endif
   if ((*script_itemize) (wtext, wlen, G_N_ELEMENTS (items), &control, NULL,
 			 items, &nitems))
     {
 #ifdef BASIC_WIN32_DEBUGGING
       if (pango_win32_debug)
-	printf ("pango-basic-win32: ScriptItemize failed\n");
+	printf ("ScriptItemize failed\n");
 #endif
       return FALSE;
     }
 
 #ifdef BASIC_WIN32_DEBUGGING
   if (pango_win32_debug)
-    printf (G_STRLOC ": ScriptItemize: %d items\n", nitems);
+    printf ("%d items:\n", nitems);
 #endif
 
   if (analysis->level % 2)
@@ -756,16 +766,21 @@ itemize_shape_and_place (PangoFont        *font,
       ABC abc;
       int script = items[item].a.eScript;
       int ng;
+      int char_offset;
 
       memset (advances, 0, sizeof (advances));
       memset (offsets, 0, sizeof (offsets));
       memset (&abc, 0, sizeof (abc));
 	  
+      /* Note that itemlen is number of wchar_t's i.e. surrogate pairs
+       * count as two!
+       */
       itemlen = items[item+1].iCharPos - items[item].iCharPos;
+      char_offset = unichar_index (wtext, items[item].iCharPos);
 
 #ifdef BASIC_WIN32_DEBUGGING
       if (pango_win32_debug)
-	printf ("  Item %d: iCharPos=%d eScript=%d (%s) %s%s%s%s%s%s%s chars %d--%d (%d)\n",
+	printf ("  Item %d: iCharPos=%d eScript=%d (%s) %s%s%s%s%s%s%s wchar_t %d--%d (%d)\n",
 		item, items[item].iCharPos, script,
 		lang_name (scripts[script]->langid),
 		scripts[script]->fComplex ? "complex" : "simple",
@@ -778,7 +793,9 @@ itemize_shape_and_place (PangoFont        *font,
 		items[item].iCharPos, items[item+1].iCharPos-1, itemlen);
 #endif
 
-      if ((*script_shape) (hdc, &script_cache[script], wtext + items[item].iCharPos, itemlen,
+      items[item].a.fRTL = analysis->level % 2;
+      if ((*script_shape) (hdc, &script_cache[script],
+			   wtext + items[item].iCharPos, itemlen,
 			   G_N_ELEMENTS (iglyphs),
 			   &items[item].a,
 			   iglyphs,
@@ -802,9 +819,10 @@ itemize_shape_and_place (PangoFont        *font,
       ng = glyphs->num_glyphs;
       pango_glyph_string_set_size (glyphs, ng + nglyphs);
       
-      set_up_pango_log_clusters (items[item].a.fRTL, itemlen, log_clusters,
+      set_up_pango_log_clusters (wtext,
+				 items[item].a.fRTL, itemlen, log_clusters,
 				 nglyphs, glyphs->log_clusters + ng,
-				 items[item].iCharPos);
+				 char_offset);
 
       if ((*script_place) (hdc, &script_cache[script], iglyphs, nglyphs,
 			   visattrs, &items[item].a,
@@ -847,7 +865,7 @@ itemize_shape_and_place (PangoFont        *font,
 #ifdef BASIC_WIN32_DEBUGGING
   if (pango_win32_debug)
     {
-      printf ("  Pango log_clusters (%d), char index:", analysis->level);
+      printf ("  Pango log_clusters (level:%d), char index:", analysis->level);
       for (glyphix = 0; glyphix < glyphs->num_glyphs; glyphix++)
 	printf ("%d ", glyphs->log_clusters[glyphix]);
       printf ("\n");
@@ -865,20 +883,18 @@ uniscribe_shape (PangoFont        *font,
 		 PangoGlyphString *glyphs)
 {
   wchar_t *wtext;
-  int wlen, i;
+  long wlen;
+  int i;
   gboolean retval = TRUE;
   HGDIOBJ old_font = NULL;
   HFONT hfont = NULL;
   LOGFONT *lf;
   SCRIPT_CACHE script_cache[100];
 
-  wtext = (wchar_t *) g_convert (text, length, "UTF-16LE", "UTF-8",
-				 NULL, &wlen, NULL);
+  wtext = (wchar_t *) g_utf8_to_utf16 (text, length, NULL, &wlen, NULL);
   if (wtext == NULL)
     return FALSE;
 
-  wlen /= 2;
-
   lf = pango_win32_font_logfont (font);
   hfont = pango_win32_font_cache_load (font_cache, lf);
   g_free (lf);
@@ -921,7 +937,6 @@ uniscribe_shape (PangoFont        *font,
 	  printf ("\n");
 	}
 #endif
-
     }
 
   g_free (wtext);
@@ -931,7 +946,7 @@ uniscribe_shape (PangoFont        *font,
   if (hfont != NULL)
     pango_win32_font_cache_unload (font_cache, hfont);
 
-  return retval;
+  return retval && glyphs->num_glyphs > 0;
 }
 
 static gboolean
@@ -940,23 +955,19 @@ text_is_simple (const char *text,
 {
   gboolean retval;
   wchar_t *wtext;
-  int wlen;
-
-  wtext = (wchar_t *) g_convert (text, length, "UTF-16LE", "UTF-8",
-				 NULL, &wlen, NULL);
+  long wlen;
 
+  wtext = (wchar_t *) g_utf8_to_utf16 (text, length, NULL, &wlen, NULL);
   if (wtext == NULL)
     return TRUE;
 
-  wlen /= 2;
-
   retval = ((*script_is_complex) (wtext, wlen, SIC_COMPLEX) == S_FALSE);
 
   g_free (wtext);
 
 #ifdef BASIC_WIN32_DEBUGGING
   if (pango_win32_debug)
-    printf ("text_is_simple: %.*s (%d chars): %s\n",
+    printf ("text_is_simple: %.*s (%ld wchar_t): %s\n",
 	    MIN (length, 10), text, wlen, retval ? "YES" : "NO");
 #endif
 
@@ -1110,20 +1121,9 @@ init_uniscribe (void)
 #endif
 } 
 
-static PangoCoverageLevel
-basic_engine_covers (PangoEngineShape *engine,
-		     PangoFont        *font,
-		     PangoLanguage    *lang,
-		     gunichar          wc)
-{
-  return PANGO_COVERAGE_EXACT;
-  return find_char (font, wc) ? PANGO_COVERAGE_EXACT : PANGO_COVERAGE_NONE;
-}
-
 static void
 basic_engine_win32_class_init (PangoEngineShapeClass *class)
 {
-  class->covers = basic_engine_covers;
   class->script_shape = basic_engine_shape;
 }
 
