@@ -38,15 +38,15 @@ typedef PangoEngineShapeClass IndicEngineFcClass ; /* No extra fields needed */
 struct _IndicEngineFc
 {
   PangoEngineShape shapeEngine;
-  PangoIndicInfo  *indicInfo;
+  const PangoIndicInfo  *indicInfo;
 };
 
 struct _PangoIndicInfo
 {
-  PangoOTTag         scriptTag; 
-  IndicOTClassTable *classTable;
-  gchar             *gsubQuarkName;
-  gchar             *gposQuarkName;
+  PangoOTTag               scriptTag; 
+  const IndicOTClassTable *classTable;
+  gchar                   *gsubQuarkName;
+  gchar                   *gposQuarkName;
 };
 
 #define ENGINE_SUFFIX "ScriptEngineFc"
@@ -122,7 +122,7 @@ static PangoEngineInfo script_engines[] = {
  * in a macro that calls a body macro that can be redefined, or by
  * putting the pointers to the PangoEngineInfo in PangoIndicInfo...
  */
-static PangoIndicInfo indic_info[] = {
+static const PangoIndicInfo indic_info[] = {
   PANGO_INDIC_INFO(deva), PANGO_INDIC_INFO(beng), PANGO_INDIC_INFO(guru),
   PANGO_INDIC_INFO(gujr), PANGO_INDIC_INFO(orya), PANGO_INDIC_INFO(taml),
   PANGO_INDIC_INFO(telu), PANGO_INDIC_INFO(knda), PANGO_INDIC_INFO(mlym),
@@ -173,7 +173,7 @@ maybe_add_GPOS_feature (PangoOTRuleset *ruleset,
 }
 
 static PangoOTRuleset *
-get_gsub_ruleset (FT_Face face, PangoIndicInfo *indic_info)
+get_gsub_ruleset (FT_Face face, const PangoIndicInfo *indic_info)
 {
   PangoOTInfo    *info = pango_ot_info_get (face);
   GQuark          ruleset_quark = g_quark_from_string (indic_info->gsubQuarkName);
@@ -217,7 +217,7 @@ get_gsub_ruleset (FT_Face face, PangoIndicInfo *indic_info)
 
 
 static PangoOTRuleset *
-get_gpos_ruleset (FT_Face face, PangoIndicInfo *indic_info)
+get_gpos_ruleset (FT_Face face, const PangoIndicInfo *indic_info)
 {
   PangoOTInfo    *info = pango_ot_info_get (face);
   GQuark          ruleset_quark = g_quark_from_string (indic_info->gposQuarkName);
@@ -250,7 +250,7 @@ get_gpos_ruleset (FT_Face face, PangoIndicInfo *indic_info)
 }
 
 static void
-set_glyphs (PangoFont *font, FT_Face face, const gunichar *wcs, gulong *tags, glong n_glyphs, PangoOTBuffer *buffer)
+set_glyphs (PangoFont *font, FT_Face face, const gunichar *wcs, gulong *tags, glong n_glyphs, PangoOTBuffer *buffer, gboolean process_zwj)
 {
   gint i;
 
@@ -260,7 +260,8 @@ set_glyphs (PangoFont *font, FT_Face face, const gunichar *wcs, gulong *tags, gl
     {
       guint glyph;
 
-      if (ZERO_WIDTH_CHAR (wcs[i]))
+      if (ZERO_WIDTH_CHAR (wcs[i]) &&
+	  (!process_zwj || wcs[i] != 0x200D))
 	glyph = 0;
       else
 	glyph = FT_Get_Char_Index (face, wcs[i]);
@@ -318,7 +319,7 @@ indic_engine_shape (PangoEngineShape *engine,
   PangoOTRuleset *gsub_ruleset = NULL, *gpos_ruleset = NULL;
   PangoOTBuffer *buffer;
   IndicEngineFc *indic_shape_engine = NULL;
-  PangoIndicInfo *indic_info = NULL;
+  const PangoIndicInfo *indic_info = NULL;
   PangoFcFont *fc_font;
   MPreFixups *mprefixups;
 
@@ -347,7 +348,9 @@ indic_engine_shape (PangoEngineShape *engine,
   
   pango_glyph_string_set_size (glyphs, n_glyphs);
   buffer = pango_ot_buffer_new (fc_font);
-  set_glyphs(font, face, wc_out, tags, n_glyphs, buffer);
+
+  set_glyphs(font, face, wc_out, tags, n_glyphs, buffer,
+	     (indic_info->classTable->scriptFlags & SF_PROCESS_ZWJ) != 0);
 
   /* do gsub processing */
   gsub_ruleset = get_gsub_ruleset (face, indic_info);
