@@ -61,7 +61,7 @@ static PangoEngineInfo script_engines[] = {
 
 /* TIS-to-Unicode glyph maps for characters 0x80-0xff
  */
-static int tis620_0[128] = {
+static const int tis620_0[128] = {
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
@@ -80,10 +80,10 @@ static int tis620_0[128] = {
     0x0e58, 0x0e59, 0x0e5a, 0x0e5b,      0,      0,      0,      0
 };
 
-static int tis620_1[128] = {
-    /**/ 0,      0,      0, 0xf88c, 0xf88f, 0xf892, 0xf895, 0xf898,
+static const int tis620_1[128] = {
+    0xf89e,      0,      0, 0xf88c, 0xf88f, 0xf892, 0xf895, 0xf898,
     0xf88b, 0xf88e, 0xf891, 0xf894, 0xf897,      0,      0, 0xf899,
-    /**/ 0,      0, 0xf884, 0xf889, 0xf885, 0xf886, 0xf887, 0xf888,
+    0xf89a,      0, 0xf884, 0xf889, 0xf885, 0xf886, 0xf887, 0xf888,
     0xf88a, 0xf88d, 0xf890, 0xf893, 0xf896,      0,      0,      0,
     /**/ 0, 0x0e01, 0x0e02, 0x0e03, 0x0e04, 0x0e05, 0x0e06, 0x0e07,
     0x0e08, 0x0e09, 0x0e0a, 0x0e0b, 0x0e0c, 0x0e0d, 0x0e0e, 0x0e0f,
@@ -96,10 +96,10 @@ static int tis620_1[128] = {
     0x0e40, 0x0e41, 0x0e42, 0x0e43, 0x0e44, 0x0e45, 0x0e46, 0x0e47,
     0x0e48, 0x0e49, 0x0e4a, 0x0e4b, 0x0e4c, 0x0e4d,      0, 0x0e4f,
     0x0e50, 0x0e51, 0x0e52, 0x0e53, 0x0e54, 0x0e55, 0x0e56, 0x0e57,
-    0x0e58, 0x0e59,      0,      0,      0,      0,      0,      0
+    0x0e58, 0x0e59,      0,      0, 0xf89b, 0xf89c, 0xf89d,      0
 };
 
-static int tis620_2[128] = {
+static const int tis620_2[128] = {
     0xf700, 0xf701, 0xf702, 0xf703, 0xf704, 0x2026, 0xf705, 0xf706,
     0xf707, 0xf708, 0xf709, 0xf70a, 0xf70b, 0xf70c, 0xf70d, 0xf70e,
     0xf70f, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014,
@@ -118,7 +118,7 @@ static int tis620_2[128] = {
     0x0e58, 0x0e59, 0x0e5a, 0x0e5b, 0xf718, 0xf719, 0xf71a,      0
 };
 
-static int lao_0[128] = {
+static const int lao_0[128] = {
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
     /**/ 0,      0,      0,      0,      0,      0,      0,      0, 
