@@ -535,7 +535,7 @@ pango_fc_convert_weight_to_fc (PangoWeight pango_weight)
   if (pango_weight < (PANGO_WEIGHT_LIGHT + PANGO_WEIGHT_NORMAL) / 2)
     return FC_WEIGHT_LIGHT;
   else if (pango_weight < (500 /* PANGO_WEIGHT_MEDIUM */ + PANGO_WEIGHT_SEMIBOLD) / 2)
-    return FC_WEIGHT_NORMAL;
+    return FC_WEIGHT_MEDIUM;
   else if (pango_weight < (PANGO_WEIGHT_SEMIBOLD + PANGO_WEIGHT_BOLD) / 2)
     return FC_WEIGHT_DEMIBOLD;
   else if (pango_weight < (PANGO_WEIGHT_BOLD + PANGO_WEIGHT_ULTRABOLD) / 2)
@@ -1177,7 +1177,7 @@ pango_fc_convert_weight_to_pango (int fc_weight)
     return PANGO_WEIGHT_ULTRALIGHT;
   else if (fc_weight < (FC_WEIGHT_LIGHT + FC_WEIGHT_REGULAR) / 2)
     return PANGO_WEIGHT_LIGHT;
-  else if (fc_weight < (FC_WEIGHT_REGULAR + FC_WEIGHT_MEDIUM) / 2)
+  else if (fc_weight < (FC_WEIGHT_REGULAR + FC_WEIGHT_DEMIBOLD) / 2)
     return PANGO_WEIGHT_NORMAL;
   /* We group the 500/MEDIUM weight with normal to reduce confusion
    *
