@@ -24,6 +24,7 @@
 #define __PANGOFT2_PRIVATE_H__
 
 #include "pangoft2.h"
+#include "pango-renderer.h"
 #include <fontconfig/fontconfig.h>
 #include <fontconfig/fcfreetype.h>
 
@@ -55,7 +56,7 @@
 
 typedef struct _PangoFT2Font      PangoFT2Font;
 typedef struct _PangoFT2GlyphInfo PangoFT2GlyphInfo;
-
+typedef struct _PangoFT2Renderer  PangoFT2Renderer;
 
 struct _PangoFT2Font
 {
@@ -79,30 +80,44 @@ struct _PangoFT2GlyphInfo
   void *cached_glyph;
 };
 
+#define PANGO_TYPE_FT2_FONT              (pango_ft2_font_get_type ())
+#define PANGO_FT2_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT, PangoFT2Font))
+#define PANGO_FT2_IS_FONT(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_FT2_FONT))
+
+GType pango_ft2_font_get_type (void);
+
 PangoFT2Font * _pango_ft2_font_new                (PangoFT2FontMap   *ft2fontmap,
 						   FcPattern         *pattern);
 FT_Library     _pango_ft2_font_map_get_library    (PangoFontMap      *fontmap);
 const char    *_pango_ft2_ft_strerror             (FT_Error           error);
 
-void *pango_ft2_font_get_cache_glyph_data    (PangoFont      *font,
-					      int             glyph_index);
-void  pango_ft2_font_set_cache_glyph_data    (PangoFont      *font,
-					      int             glyph_index,
-					      void           *cached_glyph);
-void  pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
-					      GDestroyNotify  destroy_notify);
+void *_pango_ft2_font_get_cache_glyph_data    (PangoFont      *font,
+					       int             glyph_index);
+void  _pango_ft2_font_set_cache_glyph_data    (PangoFont      *font,
+					       int             glyph_index,
+					       void           *cached_glyph);
+void  _pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
+					       GDestroyNotify  destroy_notify);
 
-void _pango_ft2_draw_rect            (FT_Bitmap   *bitmap,
-				      PangoMatrix *matrix,
-				      int          x,
-				      int          y,
-				      int          width,
-				      int          height);
-void _pango_ft2_draw_error_underline (FT_Bitmap   *bitmap,
-				      PangoMatrix *matrix,
-				      int          x,
-				      int          y,
-				      int          width,
-				      int          height);
+void _pango_ft2_draw_rect            (FT_Bitmap         *bitmap,
+				      const PangoMatrix *matrix,
+				      int                x,
+				      int                y,
+				      int                width,
+				      int                height);
+void _pango_ft2_draw_error_underline (FT_Bitmap         *bitmap,
+				      const PangoMatrix *matrix,
+				      int                x,
+				      int                y,
+				      int                width,
+				      int                height);
+
+#define PANGO_TYPE_FT2_RENDERER            (pango_ft2_renderer_get_type())
+#define PANGO_FT2_RENDERER(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_RENDERER, PangoFT2Renderer))
+#define PANGO_IS_FT2_RENDERER(object)      (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_FT2_RENDERER))
+
+GType pango_ft2_renderer_get_type    (void);
+
+PangoRenderer *_pango_ft2_font_map_get_renderer (PangoFT2FontMap *ft2fontmap);
 
 #endif /* __PANGOFT2_PRIVATE_H__ */
